/**
 * Detach.java
 *
 * @author Dj
 */

/******************************************************************************

Copyright (c) 2002, D.J.Walker-Morgan/RunState

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the RunState nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/


package com.runstate.detach;

import javax.swing.*;

public class Detach
{
		
	/**
	 * This method is called after start up.
	 *
	 * @param args   The command-line arguments.
	 *               They can be changed in the
	 *               "Project Settings" dialog in CodeGuide.
	 */
	public static void main(String[] args)
	{
		
		// Print available command-line arguments to console.
		checkArguments(args);
		
		if(lookandfeel!=null)
		{
			try
			{
				UIManager.setLookAndFeel(lookandfeel);
			}
			catch (ClassNotFoundException e)
			{
				System.err.println("Could Not Find Look and Feel classes");
				System.exit(1);
			}
			catch (UnsupportedLookAndFeelException e)
			{
				System.err.println("Could Not Find Look and Feel classes");
				System.exit(1);
			}
			catch (InstantiationException e)
			{
				System.err.println("Could Not Instansiate Look and Feel classes");}
			catch (IllegalAccessException e) { System.err.println("Illegal Access to Look and Feel classes");}
		}

		DetachController dc=new DetachController();
		
		dc.setVisible(true);
	}
	
	static String lookandfeel=null;
	/**
	 * This method prints available command-line arguments
	 * to the console.
	 */
	private static void checkArguments(String[] args)
	{
		for (int i = 0; i < args.length; i++)
		{
			if(args[i].equals("-motif"))
			{
				lookandfeel="com.sun.java.swing.plaf.motif.MotifLookAndFeel";
			}
			else if(args[i].equals("-windows"))
			{
				lookandfeel="com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
			}
			else if(args[i].equals("-lookandfeel"))
			{
				i++;
				if(i<args.length)
				{
					lookandfeel=args[i];
				}
				else
				{
					System.err.println("Need to specify an look and feel with -lookandfeel");
					System.exit(0);
				}
			}
		}
		
	}
	
}

