/**
 * Detach.java
 *
 * @author Dj
 */

/******************************************************************************

Copyright (c) 2002, D.J.Walker-Morgan/RunState

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the RunState nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/


package com.runstate.detach;


import javax.mail.*;
import javax.mail.internet.*;
import java.io.*;
import java.net.*;
import javax.swing.SwingUtilities;

import com.runstate.detach.properties.*;
import com.runstate.detach.utils.*;

public class Detacher
{
	Session session;
	Store store;
	Folder folder;
	
	public Detacher()
	{
	}
	
	
	public String scan(DetachController dc,DetachProperties dp)
	{
		int attachments=0;
		
		session=Session.getDefaultInstance(System.getProperties(),null);
		session.setDebug(false);
		
		try
		{
			store=session.getStore(dp.getProto());
		}
		catch(NoSuchProviderException nspe)
		{
			return "Unsupported provider";
		}
		
		try
		{
			store.connect(dp.getServer(),dp.getUser(),dp.getPassword());
		
			folder=store.getFolder(dp.getFolder());
		
			folder.open(Folder.READ_WRITE);
		
			final Message[] msgs=folder.getMessages();
			
			for(int i=0;i<msgs.length; i++)
			{
				Message m=msgs[i];
				Address from=m.getFrom()[0];
				String fromAddress;
				if(from instanceof InternetAddress)
				{
					fromAddress=((InternetAddress)from).getAddress();
				}
				else
				{
					fromAddress=from.toString();
				}
				if(dc!=null)
				{
					msgupdate(dc,i,0,msgs.length,"Reading "+(i+1)+" with subject "+m.getSubject() );
				}
					
				int numvalidattaches=0;
				int numinvalidattaches=0;
				
				if(m.getContent() instanceof Multipart)
				{
					Multipart multipart=(Multipart)m.getContent();
					
					attupdate(dc,0,0,multipart.getCount());
					
					for (int j=0, n=multipart.getCount(); j<n; j++)
					{
						attupdate(dc,j,0,multipart.getCount());
					
						Part part=multipart.getBodyPart(j);
						if(handlePart(dc,dp,part))
						{
							attachments++;
							numvalidattaches++;
						}
						else
						{
							numinvalidattaches++;
						}
					}
					clearattupdate(dc);
				}
				
				int mode=dp.getDeleteMode();
				
				if(mode==DetachProperties.DELETE_ALL)
				{
						m.setFlag(Flags.Flag.DELETED,true);
				}
				else if(mode==DetachProperties.DELETE_SAVED_ATTACH)
				{
					if(numvalidattaches>0)
					{
						m.setFlag(Flags.Flag.DELETED,true);
					}
				}
				else if(mode==DetachProperties.DELETE_ALL_ATTACH)
				{
					if(numvalidattaches>0 || numinvalidattaches>0)
					{
						m.setFlag(Flags.Flag.DELETED,true);
					}
				}
				else if(mode==DetachProperties.DELETE_ALL)
				{
					m.setFlag(Flags.Flag.DELETED,true);
				}
			}
			
			if(dp.getProto().equalsIgnoreCase("imap"))
			{
				folder.expunge();
			}
			
			dc.doneScanning("Scanned "+msgs.length+" messages and got "+attachments+" attachments");
		}
		catch (AuthenticationFailedException ae)
		{
			return "Bad username/password";
		}
		catch(MessagingException me)
		{
				Exception ne=me.getNextException();
					if(ne==null)
					{
						return "Error:"+me.toString();
					}
					else
					{
						if(ne instanceof UnknownHostException)
						{
							return "Error: Unkown host";
						}
						else
						{
							return "Error:"+ne.toString();
						}
					}
		}
		catch(Exception e)
		{
			return e.getMessage()+" during scan";
		}
		finally
		{
			try
			{
				if(folder!=null)
				{
					folder.close(true);
				}
				if(store!=null)
				{
					store.close();
				}
			}
			catch(MessagingException me)
			{
				return "Messaging exception when recovering from exception";
			}
		}

		return null;
	}
	
	private void msgupdate(final DetachController dc,final int curr,final int min,final int max,final String txt)
	{
		Runnable updater=new Runnable() {
					public void run() { dc.setMsgStatus(curr,min,max,txt); }
				};
				
		SwingUtilities.invokeLater(updater);
	}
	
	private void attupdate(final DetachController dc,final int curr,final int min,final int max)
	{
		Runnable updater=new Runnable() {
					public void run() { dc.setAttStatus(curr,min,max); }
				};
				
		SwingUtilities.invokeLater(updater);
	}
	
	private void clearattupdate(final DetachController dc)
	{
		Runnable updater=new Runnable() {
					public void run() { dc.clearAttStatus(); }
				};
				
		SwingUtilities.invokeLater(updater);
	}
	private void updateText(final DetachController dc,final String txt)
	{
		Runnable updater=new Runnable() {
					public void run() { dc.updateStatusText(txt); }
				};
				
		SwingUtilities.invokeLater(updater);
	}
	
	private boolean handlePart(DetachController dc,DetachProperties dp,Part part) throws MessagingException, IOException
	{
		String disposition = part.getDisposition();
		String contentType = part.getContentType();
		if (disposition == null) { // When just body
			// System.out.println("Null: "  + contentType);
			// Check if plain
			if ((contentType.length() >= 10) && (contentType.toLowerCase().substring(0, 10).equals("text/plain")))
			{
				//
				// part.writeTo(System.out);
			}
			else
			{ // Don't think this will happen
				//System.out.println("Other body: " + contentType);
				//part.writeTo(System.out);
			}
		} else if (disposition.equalsIgnoreCase(Part.ATTACHMENT))
			{
				try
				{
					updateText(dc,"Attachment: " + part.getFileName() + " : " + contentType);
					return saveFile(dp,part.getFileName(), part.getInputStream());
				}
				catch(FileNotFoundException fnfe)
				{
					return false;
				}
			}
			else if (disposition.equalsIgnoreCase(Part.INLINE))
			{
				try
				{
					updateText(dc,"Inline: " +  part.getFileName() + " : " + contentType);
					return saveFile(dp,part.getFileName(), part.getInputStream());
				}
				catch(FileNotFoundException fnfe)
				{
					return false;
				}
			}
			else
			{
				// Should never happen
				//updateText(dc,"Other: " + disposition);
			}
		
		return false;
	}
	
	private boolean saveFile(DetachProperties dp,String filename,InputStream input)
	{
		if (filename == null)
		{
			// No filename, no save.
			return false;
		}
		
		String oldfilename=filename;
		
		filename=filename.replaceAll("\\?","");
		
		if(dp.getFilter()!=null)
		{
			if(!WildcardMatch.match(dp.getFilter(),filename))
			{
				return false;
			}
		}
		
		File localdir=new File(dp.getLocaldir());
		
		if(!localdir.exists())
		{
			return false;
		}
			
				
		if(dp.isSmartPath())
		{
			int i=0;
			
			while(i<filename.length() && (filename.charAt(i)<'0' || filename.charAt(i)>'9'))
			{
				i++;
			}
		
			localdir=new File(localdir,filename.substring(0,i));
			
			if(!localdir.exists())
			{
				localdir.mkdir();
			}
		}
		
		// Do no overwrite existing file
		File file;
		
		file = new File(localdir,filename);
		
		
	
		for (int i=0; file.exists(); i++) {
			if(dp.isSkipDuplicates()) return false;
			file = new File(localdir,filename+i);
		}
		
		FileOutputStream fos;
		
		try
		{
			fos = new FileOutputStream(file);
		
			BufferedOutputStream bos = new BufferedOutputStream(fos);
		
			BufferedInputStream bis = new BufferedInputStream(input);
		
			int aByte;
			while ((aByte = bis.read()) != -1)
			{
				bos.write(aByte);
		}
			bos.flush();
			bos.close();
			bis.close();
		}
		catch(IOException e)
		{
			System.out.println("Tried to save "+filename+" and failed");
			e.printStackTrace();
			return false;
		}
			
		return true;
	}
}
