/**
 * DetachPropertiesEditor.java
 *
 * @author Dj
 */

/******************************************************************************

Copyright (c) 2002, D.J.Walker-Morgan/RunState

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the RunState nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/

package com.runstate.detach.properties;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

import com.runstate.detach.utils.*;

public class DetachPropertiesCustomizer extends JPanel implements ActionListener,ChangeListener
{
	DetachProperties dp;
	
	PropertyChangeSupport support;
	
	String[] protos={ "imap" , "pop3" };
	String[] deletemodes={ "Never",
							"Messages with valid and saved attachments",
							"All messages with attachments",
							"All messages" };
	
	JLabel namelabel=new JLabel("Name",JLabel.RIGHT);
	JSmartTextField namefield=new JSmartTextField(20);
	JLabel serverlabel=new JLabel("Server",JLabel.RIGHT);
	JSmartTextField serverfield=new JSmartTextField(20);
	JLabel protolabel=new JLabel("Protocol",JLabel.RIGHT);
	JComboBox protofield=new JComboBox(protos);
	JLabel userlabel=new JLabel("User",JLabel.RIGHT);
	JSmartTextField userfield=new JSmartTextField(20);
	JLabel passwordlabel=new JLabel("Password",JLabel.RIGHT);
	JSmartPasswordField passwordfield=new JSmartPasswordField(20);
	JLabel folderlabel=new JLabel("Folder",JLabel.RIGHT);
	JButton folderbrowsebutton=new JButton(IconUtils.getImageIcon("com/runstate/detach/properties/icons/Open.gif","Browse"));
	JSmartTextField folderfield=new JSmartTextField(20);
	JLabel localdirlabel=new JLabel("Local Directory",JLabel.RIGHT);
	JSmartTextField localdirfield=new JSmartTextField(20);
	JButton localdirbrowsebutton=new JButton(IconUtils.getImageIcon("com/runstate/detach/properties/icons/Open.gif","Browse"));
	JCheckBox skipdupesfield=new JCheckBox("Skip duplicate files");
	JCheckBox smartpathfield=new JCheckBox("Create Directories based on name (SmartPath)");
	JLabel filterlabel=new JLabel("Filter",JLabel.RIGHT);
	JSmartTextField filterfield=new JSmartTextField(20);
	JLabel deletemodelabel=new JLabel("Delete",JLabel.RIGHT);
	JComboBox deletemodefield=new JComboBox(deletemodes);
	
	JFrame parentframe;
	
	public DetachPropertiesCustomizer()
	{
		super();
	
		support=new PropertyChangeSupport(this);
		
		this.setLayout(new GridBagLayout());
		
		//namefield.setEditable(false);
		addGB(this,namelabel,0,0);
		addGB(this,namefield,1,0);
		addGB(this,serverlabel,0,1);
		addGB(this,serverfield,1,1);
		addGB(this,protolabel,0,2);
		addGB(this,protofield,1,2);
		addGB(this,userlabel,0,3);
		addGB(this,userfield,1,3);
		addGB(this,passwordlabel,0,4);
		addGB(this,passwordfield,1,4);
		addGB(this,folderlabel,0,5);
		addGB(this,folderfield,1,5);
		addGB(this,folderbrowsebutton,2,5);
		addGB(this,localdirlabel,0,6);
		addGB(this,localdirfield,1,6);
		addGB(this,localdirbrowsebutton,2,6);
		addGB(this,filterlabel,0,7);
		addGB(this,filterfield,1,7);
		addGB(this,skipdupesfield,1,8);
		addGB(this,deletemodelabel,0,9);
		addGB(this,deletemodefield,1,9);
		addGB(this,smartpathfield,1,10);
		
		ActionListener folderbrowseaction=new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				IMAPFolderSelectorDialog ifsd=new IMAPFolderSelectorDialog("Select..",dp.getServer(),dp.getUser(),dp.getPassword());
				ifsd.setLocationRelativeTo(folderbrowsebutton);
				ifsd.show();
				String name=ifsd.getSelectedFolder();
				if(name!=null)
				{
					DetachPropertiesCustomizer.this.folderfield.setText(name);
				}
			}
		};
		ActionListener localdirbrowseaction=new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				JFileChooser jfc = new JFileChooser ();
				jfc.setFileSelectionMode (JFileChooser.DIRECTORIES_ONLY);
				jfc.setCurrentDirectory(new File(localdirfield.getText()));
				int result=jfc.showDialog (null, null);
				if(result==JFileChooser.APPROVE_OPTION)
				{
					localdirfield.setText(jfc.getSelectedFile ().getAbsolutePath());
				}
			}
		};
		
		folderbrowsebutton.addActionListener(folderbrowseaction);
		localdirbrowsebutton.addActionListener(localdirbrowseaction);
	
	}
	
	public void enableListeners()
	{
		namefield.addChangeListener(this);
		serverfield.addChangeListener(this);
		protofield.addActionListener(this);
		userfield.addChangeListener(this);
		passwordfield.addChangeListener(this);
		folderfield.addChangeListener(this);
		localdirfield.addChangeListener(this);
		filterfield.addChangeListener(this);
		skipdupesfield.addActionListener(this);
		deletemodefield.addActionListener(this);
		smartpathfield.addActionListener(this);
	}
	
	public void disableListeners()
	{
		namefield.removeChangeListener(this);
		serverfield.removeChangeListener(this);
		protofield.removeActionListener(this);
		userfield.removeChangeListener(this);
		passwordfield.removeChangeListener(this);
		folderfield.removeChangeListener(this);
		localdirfield.removeChangeListener(this);
		skipdupesfield.removeActionListener(this);
		deletemodefield.removeActionListener(this);
		smartpathfield.removeActionListener(this);
	}
	
	GridBagConstraints constraints=new GridBagConstraints();
	
	private void addGB(Container jp,Component component,int x,int y)
	{
		constraints.weightx=1.0;
		constraints.weighty=1.0;
		constraints.insets=new Insets(3,3,3,3);
		constraints.fill=GridBagConstraints.BOTH;
		constraints.gridx=x;
		constraints.gridy=y;
		jp.add(component,constraints);
	}
	
	public void setObject(DetachProperties dp)
	{
		this.dp=dp;
		disableListeners();
		namefield.setText(dp.getName());
		serverfield.setText(dp.getServer());
		protofield.setSelectedItem(dp.getProto());
		userfield.setText(dp.getUser());
		passwordfield.setText(dp.getPassword());
		folderfield.setText(dp.getFolder());
		localdirfield.setText(dp.getLocaldir());
		filterfield.setText(dp.getFilter());
		deletemodefield.setSelectedIndex(dp.getDeleteMode());
		skipdupesfield.setSelected(dp.isSkipDuplicates());
		smartpathfield.setSelected(dp.isSmartPath());
		setUI(dp);
		enableListeners();
	}

	private void setUI(DetachProperties dp)
	{
		if(dp.getProto().equals("imap"))
		{
			folderfield.setEditable(true);
			folderbrowsebutton.setEnabled(true);
		}
		else if(dp.getProto().equals("pop3"))
		{
			folderfield.setEditable(false);
			folderbrowsebutton.setEnabled(false);
		}
	}
			
	public void actionPerformed(ActionEvent e)
	{
		Object src=e.getSource();
		
		if(src==protofield)
		{
			dp.setProto((String)protofield.getSelectedItem());
			setUI(dp);
			folderfield.setText(dp.getFolder());
		}
		else if(src==deletemodefield)
		{
			// This is a hack... I know the props are 0/1/2/3 ATM but I'll need to add a mapping
			dp.setDeleteMode(deletemodefield.getSelectedIndex());
		}
		else if(src==skipdupesfield)
		{
			dp.setSkipDuplicates(skipdupesfield.isSelected());
		}
		else if(src==smartpathfield)
		{
			dp.setSmartPath(smartpathfield.isSelected());
		}
	}
	
	public void stateChanged(ChangeEvent e)
	{
		Object src=e.getSource();
		
		if(src==namefield)
		{
			dp.setName(namefield.getText());
		}
		else if(src==serverfield)
		{
			dp.setServer(serverfield.getText());
		}
		else if(src==userfield)
		{
			dp.setUser(userfield.getText());
		}
		else if(src==passwordfield)
		{
			dp.setPassword(new String(passwordfield.getPassword()));
		}
		else if(src==folderfield)
		{
			dp.setFolder(folderfield.getText());
		}
		else if(src==localdirfield)
		{
			dp.setLocaldir(localdirfield.getText());
		}
		else if(src==filterfield)
		{
			dp.setFilter(filterfield.getText());
		}
	
	}
	
	
}
