/**
 * JFocusTextField.java
 *
 * @author Dj
 */

/******************************************************************************

Copyright (c) 2002, D.J.Walker-Morgan/RunState

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the RunState nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/


package com.runstate.detach.utils;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

import java.util.ArrayList;

public class JSmartPasswordField extends JPasswordField implements DocumentListener
{
	public JSmartPasswordField(int width)
	{
		super(width);
		Document doc=this.getDocument();
		doc.addDocumentListener(this);
	}
	
	
	private ArrayList mChangeListeners=new ArrayList(3);
	
	/**
       * Adds a change listener, that will be notified when the text in the
       * text field is changed. The ChangeEvent that will be passed
       * to registered listeners will contain this object as the source, allowing
       * the new text to be extracted using the {@link #getText() getText} method.
       *
       * @param pChangeListener The listener to add
       */
      public void addChangeListener(ChangeListener pChangeListener)
      {
          mChangeListeners.add(pChangeListener);
      }
  
      /**
       * Removes a change listener.
       *
       * @param pChangeListener The change listener to remove.
       */
      public void removeChangeListener(ChangeListener pChangeListener)
      {
          mChangeListeners.remove(pChangeListener);
      }
  
      /**
       * Notify all registered change listeners that the
       * text in the text field has changed.
       */
      private void notifyChangeListeners()
      {
          ChangeEvent ce = new ChangeEvent(this);
          for (int index = 0; index < mChangeListeners.size(); index ++)
          {
              ((ChangeListener)mChangeListeners.get(index)).stateChanged(ce);
          }
      }
  
	public void insertUpdate(DocumentEvent e)
	{
		notifyChangeListeners();
	}
	
	public void removeUpdate(DocumentEvent e)
	{
		notifyChangeListeners();
	}
	
	public void changedUpdate(DocumentEvent e)
	{
		notifyChangeListeners();
	}
	
	
}

