/**
 * WildcardMatch.java
 *
 * @author Dj
 */

/******************************************************************************

Copyright (c) 2002, D.J.Walker-Morgan/RunState

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the RunState nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/


package com.runstate.detach.utils;

public class WildcardMatch
{

   // Any single character
   public static final char ANY = '?';
   // Zero or more characters
   public static final char MORE = '*';
   // Relevant under Windows
   public static final String DOS = "*.*";

   public static boolean match(String wild, String text)
   {
      if (wild.length() == 0) return true;
      if (text.length() == 0) return false;
      
	  int j = 0;
      for (int i = 0; i < wild.length(); i++)
      {
         if (j > text.length()) return false;
         // We have a '?' wildcard, match any character
         else if (wild.charAt(i) == ANY)
         {
            j++;
         }
         // We have a '*' wildcard, check for
         // a match in the tail
         else if (wild.charAt(i) == MORE)
         {
            for (int f = j; f < text.length(); f++)
            {
               if (match(wild.substring(i + 1),
                  text.substring(f)))
                  return true;
            }
            return false;
         }
         // Both characters match, case insensitive
         else if (j < text.length() &&
            Character.toUpperCase(wild.charAt(i)) !=
            Character.toUpperCase(text.charAt(j)))
         {
            return false;
         }
         else j++;
      }
      if (j == text.length()) return true;
      else return false;
   }
}

